﻿Imports System.Windows.Forms
Imports System.Drawing

Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False
    Friend RecogTextFile As String = "Colors_ENG"

    Friend GCI As Globalization.CultureInfo = Globalization.CultureInfo.InvariantCulture

    ' =======================================================================================================
    '   APP TITLE AND VERSION (ALL FIELDS)
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal RemoveThereminoPrefix As Boolean = False) As String
        Dim appname As String = IO.Path.GetFileNameWithoutExtension(Application.ExecutablePath)
        Dim Title As String = Replace(appname, "_", " ")
        If RemoveThereminoPrefix Then Title = Replace(Title, "Theremino ", "")
        Return Title + " - V" + GetVersionString()
    End Function

    ' =======================================================================================================
    '   GET VERSION (ALSO TRIPLE OR QUADRUPLE FIELDS)
    ' =======================================================================================================
    Friend Function GetVersionString() As String
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        GetVersionString = s(0) + "." + s(1)
        If s(2) <> "0" Then
            GetVersionString += "." + s(2)
            If s(3) <> "0" Then
                GetVersionString += "." + s(3)
            End If
        End If
    End Function


    ' =======================================================================================
    '  FORM FUNCTIONS
    ' =======================================================================================
    Friend Sub LimitFormPosition_CompletelyVisible(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormCompletelyVisible(f)
    End Sub

    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
    End Sub

    Private SB As Rectangle = New Rectangle(Integer.MaxValue, Integer.MaxValue, Integer.MinValue, Integer.MinValue)

    Private Sub GetMaxScreenBounds()
        For Each s As Screen In System.Windows.Forms.Screen.AllScreens
            SB = Rectangle.Union(SB, s.WorkingArea)
        Next
    End Sub

    Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)         ' not more than a maximized window
            .Height = Math.Min(.Height, SB.Height)      ' not more than a maximized window
            .Width = Math.Max(.Width, 32)               ' at least 32x24
            .Height = Math.Max(.Height, 24)             ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - .Width)  ' not beyond the right border
            .Top = Math.Min(.Top, SB.Bottom - .Height)  ' not beyond the bottom border
            .Left = Math.Max(.Left, SB.Left)            ' at least at the left border
            .Top = Math.Max(.Top, SB.Top)               ' at least at the top border
        End With
    End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)             ' not more than VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, SB.Height)          ' not more than VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)                   ' at least 32x24
            .Height = Math.Max(.Height, 24)                 ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - 200)         ' not beyond right border - 200 pixels
            .Top = Math.Min(.Top, SB.Bottom - 150)          ' not beyond bottom border - 150 pixels
            .Left = Math.Max(.Left, SB.Left + 200 - .Width) ' at least at left border + 200 pixels
            .Top = Math.Max(.Top, SB.Top)                   ' at least at top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================
    Private Function TabString(ByVal Name As String, _
                               Optional ByVal Value As Double = Double.NaN, _
                               Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt, GCI)
        End If
    End Function

    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function

    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function

    Private Function Val_Single(ByVal l As String) As Single
        Return CSng(Val(l.Replace(",", ".")))
    End Function

    Private Function Val_Double(ByVal l As String) As Double
        Return Val(l.Replace(",", "."))
    End Function

    Private Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function

    Private Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function

    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function


    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================
    Friend Sub Save_INI()
        '
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
        Dim f As System.IO.StreamWriter = Nothing
        Try
            f = IO.File.CreateText(iniFileName)
            '
            f.WriteLine(" Program Params")
            f.WriteLine("===========================================")
            ' ------------------------------------------------------------------------------ FORM BOUNDS
            Dim r As Rectangle
            r = GetFormRectangle(Form1)
            f.WriteLine(TabString("Form1_Top", r.Top))
            f.WriteLine(TabString("Form1_Left", r.Left))
            f.WriteLine(TabString("Form1_Width", r.Width))
            f.WriteLine(TabString("Form1_Height", r.Height))
            f.WriteLine(TabString("Form1_WindowState", Form1.WindowState))
            '
            f.WriteLine(TabString(""))
            f.WriteLine(TabString(" Slots"))
            f.WriteLine("===========================================")
            f.WriteLine(TabString("RecogTextFile", RecogTextFile))
            f.WriteLine(TabString("Running", Form1.Running))
        Catch
        End Try
        Try
            f.Close()
        Catch
        End Try

        SaveRecogFile()
    End Sub

    Friend Sub Load_INI()
        ' ------------------------------------------------------------------------------- defaults
        '
        ' -------------------------------------------------------------------------------
        ' ------------------------------------------------------------------------------- 
        ' With "Resume Next" subsequent parameters are loaded and f.Close() is executed
        ' -------------------------------------------------------------------------------
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' -------------------------------------------------------------------------------
        Dim l As String
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
        If My.Computer.FileSystem.FileExists(iniFileName) Then

            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)

            Do While Not f.EndOfStream
                l = f.ReadLine()
                Select Case ExtractParamName(l)
                    Case "Form1_Top" : Form1.Top = Val_Int(l)
                    Case "Form1_Left" : Form1.Left = Val_Int(l)
                    Case "Form1_Width" : Form1.Width = Val_Int(l)
                    Case "Form1_Height" : Form1.Height = Val_Int(l)
                    Case "Form1_WindowState" : Form1.WindowState = CType((Val(l)), FormWindowState)
                        ' ----------------------------------------------
                    Case "RecogTextFile" : RecogTextFile = l
                    Case "Running" : Form1.Running = l = "True"
                End Select
            Loop
            f.Close()
        End If
        LimitFormPosition(Form1)
        ' -----------------------------------------------
        ReadRecogFile()
    End Sub


    ' ==================================================================================================
    '  SAVE LOAD -- RECOG AND SPEECH FILES
    ' ==================================================================================================
    Private RecogPath As String = Application.StartupPath + "\Recog_Files\"
    Friend Sub ReadRecogFile()
        Dim fname As String = RecogPath + RecogTextFile
        If My.Computer.FileSystem.FileExists(fname) Then
            Form1.txt_ValidWords.Text = ReadTextFile(fname)
            Form1.txt_ValidWords.SelectionLength = 0
            Form1.lbl_Recog.Text = RecogTextFile
        End If
    End Sub
    Friend Sub SaveRecogFile()
        If Form1.txt_ValidWords.Text = "" Then Return
        Dim fname As String = RecogPath + RecogTextFile
        WriteTextFile(fname, Form1.txt_ValidWords.Text)
        Form1.lbl_Recog.Text = RecogTextFile
    End Sub

    Friend Function RecogFileExists(ByVal fname As String) As Boolean
        fname = RecogPath + fname
        Return IO.File.Exists(fname)
    End Function

    Friend Sub WriteTextFile(ByVal fname As String, ByVal text As String)
        'text = Replace(text, vbLf, vbCrLf)
        Dim writer As System.IO.StreamWriter
        writer = New System.IO.StreamWriter(fname)
        writer.Write(text)
        writer.Close()
    End Sub

    Friend Function ReadTextFile(ByVal fname As String) As String
        Dim reader As System.IO.TextReader
        reader = New System.IO.StreamReader(fname)
        Dim text As String = reader.ReadToEnd
        reader.Close()
        'text = Replace(text, vbLf, vbCrLf)
        'text = Replace(text, vbCrLf, vbLf)
        'text = Replace(text, vbCr, vbLf)
        'text = Replace(text, vbLf, "")
        'text = Replace(text, vbCr, "")
        Return text
    End Function

    Friend Sub RecogSave()
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.InitialDirectory = RecogPath
        sfd.DefaultExt = ".txt"
        sfd.AddExtension = True
        sfd.FileName = RecogTextFile
        sfd.Filter = "Recognition text file (*.txt)|*txt"
        '
        SendKeys.Send("+{TAB}+{TAB}" + sfd.FileName + "{TAB}{TAB}") ' file selected in the list
        'SendKeys.Send("{HOME}{END}^A") ' name visible and selected
        SendKeys.Send("{HOME}{END}")   ' name visible and unselected
        '
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            RecogTextFile = IO.Path.GetFileName(sfd.FileName)
            SaveRecogFile()
        End If
    End Sub

    Friend Sub RecogLoad()
        Dim ofd As OpenFileDialog = New OpenFileDialog()
        ofd.InitialDirectory = RecogPath
        ofd.Multiselect = True
        ofd.Filter = "Recognition text file (*.txt)|*txt"
        ofd.DefaultExt = ".txt"
        ofd.FileName = RecogTextFile
        '
        SendKeys.Send("+{TAB}+{TAB}" + ofd.FileName + "{TAB}{TAB}") ' file selected in the list
        'SendKeys.Send("{HOME}{END}^A") ' name visible and selected
        SendKeys.Send("{HOME}{END}")   ' name visible and unselected
        '
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            RecogTextFile = IO.Path.GetFileName(ofd.FileName)
            ReadRecogFile()
        End If
    End Sub

    ' ==============================================================
    '  REPLACE MULTIPLE SPACES and TABS and TRIM
    ' ============================================================== 
    Friend Function ReplaceMultipleSpacesAndTrim(ByVal s As String) As String
        s = s.Replace(vbTab, " ")
        While s.Contains("  ")
            s = s.Replace("  ", " ")
        End While
        Return s.Trim
    End Function

    ' ==============================================================
    '  REMOVE COMMENTS
    ' ============================================================== 
    Friend Function RemoveComments(ByVal s As String) As String
        Dim i As Int32 = s.IndexOf("'")
        If i = 0 Then s = ""
        If i > 0 Then
            i = s.Length
            Dim InsideString As Boolean = False
            For j As Int32 = 0 To s.Length - 1
                Select Case s(j)
                    Case """"c : InsideString = Not InsideString
                    Case "'"c
                        If Not InsideString Then
                            i = j
                            s = s.Remove(i).TrimEnd
                            Exit For
                        End If
                End Select
            Next
        End If
        Return s
    End Function

    ' ==============================================================
    '  TextBox Helpers
    ' ==============================================================
    Friend Sub TextBox_SelectLine(ByVal tbox As TextBox, ByVal lineindex As Int32)
        Dim p1 As Int32 = tbox.GetFirstCharIndexFromLine(lineindex)
        If p1 < 0 Then Return
        Dim p2 As Int32 = tbox.Text.IndexOf(vbLf, p1)
        If p2 <= 0 Then p2 = tbox.Text.Length
        tbox.SelectionStart = p1
        tbox.SelectionLength = p2 - p1
        tbox.ScrollToCaret()
    End Sub

    Friend Sub TextBox_SelectWholeLine(ByVal tbox As TextBox)
        Dim p1 As Int32 = tbox.GetFirstCharIndexOfCurrentLine()
        If p1 < 0 Then Return
        Dim p2 As Int32 = tbox.Text.IndexOf(vbLf, p1)
        If p2 <= 0 Then p2 = tbox.Text.Length
        tbox.SelectionStart = p1
        tbox.SelectionLength = p2 - p1
    End Sub


    ' =============================================================================================
    '  WINDOWS SCHEDULER PRECISION
    ' ============================================================================================= 
    Private Declare Function timeBeginPeriod Lib "winmm.dll" (ByVal uPeriod As Int32) As Int32
    Private Declare Function timeEndPeriod Lib "winmm.dll" (ByVal uPeriod As Int32) As Int32
    Friend Sub SchedulerMaxPrecision()
        If OperatingSystemIsWindows Then
            timeBeginPeriod(1)
        End If
    End Sub
    Friend Sub SchedulerDefaultPrecision()
        If OperatingSystemIsWindows Then
            timeEndPeriod(1)
        End If
    End Sub


End Module
